// History.java
// $Id: History.java,v 1.3 1999/11/15 16:48:07 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
 
package org.w3c.jwput.gui;

import java.io.*;

import java.util.LinkedList;
import java.util.ListIterator;
import java.util.NoSuchElementException;
import java.util.Properties;

import javax.swing.JComboBox;

import org.w3c.util.XMLProperties;

/**
 * History is a set of String (associated to a JComboBox)
 * @version $Revision: 1.3 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class History {
    
    public static final String COUNT_P = "count";

    protected LinkedList list    = null;

    protected int        maxsize = -1;

    protected int        count   = 0;

    protected String     title   = null;

    protected File       file    = null;

    LinkedList listeners = null;

    /**
     * Put the History elements in the given ComboBox.
     * @param combo the JComboBox to initialize
     */
    public synchronized void initialize(JComboBox combo) {
	ListIterator iterator = list.listIterator(0);
	while (iterator.hasNext()) {
	    String item = (String)iterator.next();
	    combo.addItem(item);
	}
    }

    /**
     * Add an HistoryListener
     * @param listener the History listener
     * @see HistoryListener
     */
    public void addHistoryListener(HistoryListener listener) {
	if (listeners == null)
	    listeners = new LinkedList();
	listeners.addLast(listener);
    }

    /**
     * Remove an HistoryListener
     * @param listener the History listener
     * @see HistoryListener
     */
    public void removeHistoryListener(HistoryListener listener) {
	if (listeners != null)
	    listeners.remove(listener);
    }

    protected void fireItemAddedHistoryEvent(String item) {
	HistoryEvent e = new HistoryEvent(item, this);
	ListIterator iterator = listeners.listIterator(0);
	while (iterator.hasNext()) {
	    HistoryListener listener = (HistoryListener)iterator.next();
	    listener.ItemAdded(e);
	}
    }

    protected void fireItemRemovedHistoryEvent(String item) {
	HistoryEvent e = new HistoryEvent(item, this);
	ListIterator iterator = listeners.listIterator(0);
	while (iterator.hasNext()) {
	    HistoryListener listener = (HistoryListener)iterator.next();
	    listener.ItemRemoved(e);
	}
    }

    /**
     * Add this object to the History and remove the older element of the
     * History if maxsize is reached.
     */
    public synchronized void add(String object) {
	if (list.remove(object)) {
	    fireItemRemovedHistoryEvent(object);
	    count--;
	}
	list.addFirst(object);
	fireItemAddedHistoryEvent(object);
	if (count < maxsize) {
	    count++;
	} else {
	    String item = (String)list.removeLast();
	    fireItemRemovedHistoryEvent(item);
	}
    }

    /**
     * Get the history (newer first)
     * @return an Object array
     */    
    public synchronized Object[] getHistory() {
	String array[] = new String[count];
	list.toArray(array);
	return array;
    }

    /**
     * Save the current history
     * @exception IOException if an IO error occured
     */
    public synchronized History save() 
	throws IOException
    {
	Properties props = new XMLProperties();
	int idx = 0;
	try {
	    ListIterator iterator = list.listIterator(0);
	    while (iterator.hasNext())
		props.setProperty(String.valueOf(idx++),
				  (String)iterator.next());
	} catch (IndexOutOfBoundsException ex) {
	    count = idx+1;
	}
	props.setProperty(COUNT_P, String.valueOf(count));
	//now save in the file
	OutputStream out = 
	    new BufferedOutputStream(new FileOutputStream(file));
	props.store(out, title);
	try {
	    out.close();
	} catch (IOException ex) {
	    ex.printStackTrace();
	}	    
	return this;
    }

    /**
     * Load the history
     * @exception IOException if an IO error occured
     */
    public synchronized History load() 
	throws IOException
    {
	XMLProperties props = new XMLProperties();
	try {
	    props.load(file);
	    count = Integer.parseInt(props.getProperty(COUNT_P));
	    for (int i = 0 ; i < count ; i++)
		list.addLast(props.getProperty(String.valueOf(i)));
	} catch (FileNotFoundException ex) {
	    //load nothing
	    count = 0;
	}
	return this;
    }

    public History(int maxsize, String title, File file) {
	this.maxsize = maxsize;
	this.title   = title;
	this.file    = file;
	this.count   = 0;
	this.list    = new LinkedList();
    }

}
