// StatusBar.java
// $Id: StatusBar.java,v 1.2 1999/11/24 10:25:44 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
package org.w3c.jwput.gui;

import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;

import java.awt.Dimension;

/**
 * A classical status bar
 * @version $Revision: 1.2 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class StatusBar extends JPanel {

    JLabel  statusl   = null;
    JButton buttons[] = null;
    String  descr[]   = null;

    /**
     * Constructor.<p>
     * <b>Note</b>: icons, commands and descr MUST have the same size.
     * @param icons The Buttons Icons (on the left)
     * @param commands The actions commands
     * @param descr The actions descriptions
     * @param status The default status
     */
    public StatusBar(Icon icons[], 
		     String commands[],
		     String descr[],
		     String status) 
    {
	super(new BorderLayout(0,0));
	this.descr = descr;

	statusl = new JLabel(status);
	statusl.setBorder(BorderFactory.createLoweredBevelBorder());
	if (icons != null) {
	    int len = icons.length;
	    JPanel p = new JPanel(new GridLayout(1, len));
	    buttons = new JButton[len];
	    MouseAdapter ma = new MouseAdapter() {
		public void mouseEntered(MouseEvent e) {
		    Object source = e.getSource();
		    for (int i = 0 ; i < buttons.length ; i++) {
			if (source == buttons[i]) {
			    setStatus(StatusBar.this.descr[i]);
			}
		    }
		}
		public void mouseExited(MouseEvent e) {
		    setStatus("");
		}
	    };
	    for (int i = 0 ; i < len ; i++) {
		buttons[i] = new JButton(icons[i]);
		buttons[i].setActionCommand(commands[i]);
		buttons[i].setMargin(new Insets(0, 0, 0, 0));
		buttons[i].addMouseListener(ma);
		p.add(buttons[i]);
	    }
	    add(p, "West");
	}
	add(statusl, "Center");
    }

    public void addActionListener(ActionListener listener) {
	if (buttons != null) {
	    for (int i = 0 ; i < buttons.length ; i++) {
		buttons[i].addActionListener(listener);
	    }
	}
    }
    
    /**
     * Set the Status
     * @param text the status
     */
    public void setStatus(String text) {
	statusl.setText(text);
    }

    /**
     * Clear the status bar
     */
    public void clearStatus() {
	statusl.setText("");
    }

    /**
     * Get the current status
     * @return the status
     */
    public String getStatus() {
	return statusl.getText();
    }

}
